///////////////////////////////////////////////////////////////////
// NetLogin.cpp
// 720020-1166x
//
// Applied Data Systems
//
// This sample shows how to programmatically set the device's network
// user credentials and authenticate with a server.
//
// History
// -------
// 720020-1166: Dec 8, 2003 (ctacke)
//  - Initial implementation
///////////////////////////////////////////////////////////////////

#include <windows.h>
#include <credmgr.h>
#include <sspi.h>
#include <Pkfuncs.h>

#define ARRAYSIZEOF(x)	(sizeof(x) / sizeof((x)[0]))

BOOL SaveLogin(TCHAR *szUser, TCHAR*szPass, TCHAR*szDomain);

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	// set username, password and domain
	TCHAR *lpszDomain = _T("MyDomain.net");
	TCHAR *lpszUser = _T("Username");
	TCHAR *lpszPass = _T("password");

	SaveLogin(lpszUser, lpszPass, lpszDomain);

	return 0;
}

BOOL SaveLogin(TCHAR *szUser, TCHAR*szPass, TCHAR*szDomain)
{
    PWSTR Packages[] = {L"NTLM", L"KERBEROS"};
    BOOL fRet = TRUE;
    int i;
	SECURITY_STATUS ss;
	DWORD cchDomain, cchUser,cchPassword;

	// use kernel mode
	SetKMode(TRUE);

	// get lengths of strings
	cchUser = _tcslen(szUser);
	cchDomain = _tcslen(szDomain);
	cchPassword = _tcslen(szPass);

	BOOL fSuccess = FALSE;

	// create and initialize our Credential
	CECREDENTIAL credential;
	credential.Flags = 0;
	credential.Type = CRED_TYPE_DOMAIN_PASSWORD;
	credential.CredentialBlobSize = 0;
	credential.CredentialBlob = NULL;
	credential.Persist = CRED_PERSIST_LOCAL_MACHINE;
	credential.UserName = 	(PWCHAR)LocalAlloc(0, (cchUser + cchDomain + 2) * sizeof(TCHAR));

	PTCHAR pch = credential.UserName;
	if (cchDomain)
	{
		memcpy(credential.UserName, szDomain, cchDomain * sizeof(TCHAR));
		credential.UserName[cchDomain] = '\\';
		pch = credential.UserName + cchDomain + 1;
	}
	memcpy(pch, szUser, cchUser * sizeof(TCHAR));
	pch[cchUser] = '\0';

	// write our credential info locally
	fSuccess = CeCredWrite(NULL, &credential);
	if (!fSuccess)
	{
		DEBUGMSG(TRUE, (TEXT("Failed to login %s\r\n"),credential.UserName));
		SetKMode(FALSE);
		return FALSE;
	}
	else
	{
		DEBUGMSG(TRUE, (TEXT("Successfully logged in %s\r\n"), credential.UserName));
	}
	LocalFree(credential.UserName);


	SEC_WINNT_AUTH_IDENTITY_W authIdentity = {szUser, cchUser, szDomain, cchDomain, szPass, cchPassword, SEC_WINNT_AUTH_IDENTITY_UNICODE};
	// authenticate with the server
    __try
    {
        for (i = 0; i < ARRAYSIZEOF(Packages); i++)
        {
            CredHandle hCred;
            TimeStamp Lifetime;

			ss = AcquireCredentialsHandle (
							NULL,	// principal
							Packages[i],
							SECPKG_CRED_OUTBOUND,
							NULL,	// LOGON id
							&authIdentity,
							NULL,	// get key fn
							NULL,	// get key arg
							&hCred,
							&Lifetime);

			if(ss == SEC_E_OK)
			{
				DEBUGMSG(TRUE, (TEXT("Login successful!")));
				fRet = TRUE;
				FreeCredentialsHandle(&hCred);
				break;
			}
			else
			{
				DEBUGMSG(TRUE, (TEXT("AcquireCredentialsHandle failed with 0x%08x"), ss));
				SetKMode(FALSE);
				return FALSE;
			}
        }
	}
	__except(1)
	{
		DEBUGMSG(TRUE, (TEXT("AcquireCredentialsHandle failed")));
		SetKMode(FALSE);
		return FALSE;
	}

	ZeroMemory(szPass, sizeof(szPass));

	// return to user mode
	SetKMode(FALSE);
	return TRUE;
}
